@@ -275,7 +275,7 @@ class CephBackupDriver(driver.BackupDriver):
         """Returns an error if prerequisites aren't met."""
         if rados is None or rbd is None:
             msg = _('rados and rbd python libraries not found')
-            raise exception.BackupDriverException(message=msg)
+            raise exception.BackupDriverException(reason=msg)
 
         for attr in ['backup_ceph_user', 'backup_ceph_pool',
                      'backup_ceph_conf']:
@@ -631,16 +631,16 @@ class CephBackupDriver(driver.BackupDriver):
                                         base_name, read_only=True))
         try:
             snaps = base_rbd.list_snaps()
-
-            if snaps is None:
-                return False
-
-            for snap in snaps:
-                if snap['name'] == snap_name:
-                    return True
         finally:
             base_rbd.close()
 
+        if snaps is None:
+            return False
+
+        for snap in snaps:
+            if snap['name'] == snap_name:
+                return True
+
         return False
 
     def _backup_rbd(self, backup, volume_file, volume_name, length):
@@ -1023,16 +1023,8 @@ class CephBackupDriver(driver.BackupDriver):
                 base_image.close()
 
         if adjust_size:
-            with eventlet.tpool.Proxy(rbd_driver.RADOSClient(self,
-                                      src_pool)) as client:
-                restore_vol_encode = utils.convert_str(restore_vol)
-                dest_image = eventlet.tpool.Proxy(self.rbd.Image(client.ioctx,
-                                                  restore_vol_encode))
-                try:
-                    LOG.debug("Adjusting restore vol size")
-                    dest_image.resize(adjust_size)
-                finally:
-                    dest_image.close()
+            LOG.debug("Adjusting restore vol size")
+            restore_vol.rbd_image.resize(adjust_size)
 
     def _diff_restore_rbd(self, backup, restore_file, restore_name,
                           restore_point, restore_length):
@@ -1062,7 +1054,7 @@ class CephBackupDriver(driver.BackupDriver):
         # we will need to resize it after the diff import since import-diff
         # appears to shrink the target rbd volume to the size of the original
         # backup volume.
-        self._check_restore_vol_size(backup, restore_name, restore_length,
+        self._check_restore_vol_size(backup, restore_file, restore_length,
                                      rbd_pool)
 
         LOG.debug("Restore transfer completed in %.4fs",
