@@ -43,6 +43,7 @@ from oslo_service import loopingcall
 from oslo_service import periodic_task
 from oslo_utils import excutils
 from oslo_utils import importutils
+from oslo_utils import timeutils
 import six
 
 from cinder.backup import driver
@@ -90,6 +91,12 @@ CONF.register_opts(backup_manager_opts)
 CONF.import_opt('use_multipath_for_image_xfer', 'cinder.volume.driver')
 CONF.import_opt('num_volume_device_scan_tries', 'cinder.volume.driver')
 QUOTAS = quota.QUOTAS
+MAPPING = {
+    # Module name "google" conflicts with google library namespace inside the
+    # driver when it imports google.auth
+    'cinder.backup.drivers.google': 'cinder.backup.drivers.gcs',
+}
+SERVICE_PGRP = '' if os.name == 'nt' else os.getpgrp()
 
 
 # TODO(geguileo): Once Eventlet issue #432 gets fixed we can just tpool.execute
@@ -112,24 +119,14 @@ class BackupManager(manager.ThreadPoolManager):
         super(BackupManager, self).__init__(*args, **kwargs)
         self.is_initialized = False
         self._set_tpool_size(CONF.backup_native_threads_pool_size)
-
-    @property
-    def driver_name(self):
-        """This function maps old backup services to backup drivers."""
-
-        return self._map_service_to_driver(CONF.backup_driver)
-
-    def _map_service_to_driver(self, service):
-        """Maps services to drivers."""
-
-        if service in mapper:
-            msg = ("Using legacy backup service configuration like "
-                   "cinder.backup.services.* is deprecated and "
-                   "will be removed in the 'R' release. Please use "
-                   "the cinder.backup.drivers.* method instead.")
-            versionutils.report_deprecated_feature(LOG, msg)
-            return mapper[service]
-        return service
+        self._process_number = kwargs.get('process_number', 1)
+        self.driver_name = CONF.backup_driver
+        if self.driver_name in MAPPING:
+            new_name = MAPPING[self.driver_name]
+            LOG.warning('Backup driver path %s is deprecated, update your '
+                        'configuration to the new path %s',
+                        self.driver_name, new_name)
+            self.driver_name = new_name
 
     def get_backup_driver(self, context):
         driver = None
@@ -194,7 +191,18 @@ class BackupManager(manager.ThreadPoolManager):
         self.backup_rpcapi = backup_rpcapi.BackupAPI()
         self.volume_rpcapi = volume_rpcapi.VolumeAPI()
 
+    @utils.synchronized('cleanup_incomplete_backups_%s' % SERVICE_PGRP,
+                        external=True, delay=0.1)
     def _cleanup_incomplete_backup_operations(self, ctxt):
+        # Only the first launched process should do the cleanup, the others
+        # have waited on the lock for the first one to finish the cleanup and
+        # can now continue with the start process.
+        if self._process_number != 1:
+            LOG.debug("Process #%s %sskips cleanup.",
+                      self._process_number,
+                      '(pgid=%s) ' % SERVICE_PGRP if SERVICE_PGRP else '')
+            return
+
         LOG.info("Cleaning up incomplete backup operations.")
 
         # TODO(smulcahy) implement full resume of backup and restore
@@ -301,6 +309,10 @@ class BackupManager(manager.ThreadPoolManager):
         try:
             temp_snapshot = objects.Snapshot.get_by_id(
                 ctxt, backup.temp_snapshot_id)
+            # We may want to consider routing those calls through the
+            # cinder API.
+            temp_snapshot.status = fields.SnapshotStatus.DELETING
+            temp_snapshot.save()
             self.volume_rpcapi.delete_snapshot(ctxt, temp_snapshot)
         except exception.SnapshotNotFound:
             LOG.debug("Could not find temp snapshot %(snap)s to clean "
@@ -414,8 +426,9 @@ class BackupManager(manager.ThreadPoolManager):
 
         # Restore the original status.
         if snapshot_id:
-            self.db.snapshot_update(context, snapshot_id,
-                                    {'status': fields.BackupStatus.AVAILABLE})
+            self.db.snapshot_update(
+                context, snapshot_id,
+                {'status': fields.SnapshotStatus.AVAILABLE})
         else:
             self.db.volume_update(context, volume_id,
                                   {'status': previous_status,
@@ -475,12 +488,12 @@ class BackupManager(manager.ThreadPoolManager):
                 if (isinstance(device_path, six.string_types) and
                         not os.path.isdir(device_path)):
                     if backup_device.secure_enabled:
-                        with open(device_path) as device_file:
+                        with open(device_path, 'rb') as device_file:
                             updates = backup_service.backup(
                                 backup, tpool.Proxy(device_file))
                     else:
                         with utils.temporary_chown(device_path):
-                            with open(device_path) as device_file:
+                            with open(device_path, 'rb') as device_file:
                                 updates = backup_service.backup(
                                     backup, tpool.Proxy(device_file))
                 # device_path is already file-like so no need to open it
@@ -508,15 +521,18 @@ class BackupManager(manager.ThreadPoolManager):
         if not backup:
             return True
 
-        # TODO(tommylikehu): We upgraded the 'driver_name' from module to
-        # class name, so we use 'startswith' here to match two namings,
-        # this can be replaced with equal sign during next release (Rocky).
-        # The 'driver_name' varies depending on the user settings, so a
-        # two-way judgment is required.
-        if self.driver_name.startswith(backup) or backup.startswith(
-                self.driver_name):
+        # TODO(tommylikehu): We upgraded the 'driver_name' from module
+        # to class name, so we use 'in' here to match two namings,
+        # this can be replaced with equal sign during next
+        # release (Rocky).
+        if self.driver_name.startswith(backup):
             return True
 
+        # We support renaming of drivers, so check old names as well
+        for key, value in MAPPING.items():
+            if key.startswith(backup) and self.driver_name.startswith(value):
+                return True
+
         return False
 
     def restore_backup(self, context, backup, volume_id):
@@ -531,17 +547,22 @@ class BackupManager(manager.ThreadPoolManager):
         backup.host = self.host
         backup.save()
 
-        expected_status = 'restoring-backup'
-        actual_status = volume['status']
-        if actual_status != expected_status:
+        expected_status = [fields.VolumeStatus.RESTORING_BACKUP,
+                           fields.VolumeStatus.CREATING]
+        volume_previous_status = volume['status']
+        if volume_previous_status not in expected_status:
             err = (_('Restore backup aborted, expected volume status '
                      '%(expected_status)s but got %(actual_status)s.') %
-                   {'expected_status': expected_status,
-                    'actual_status': actual_status})
+                   {'expected_status': ','.join(expected_status),
+                    'actual_status': volume_previous_status})
             backup.status = fields.BackupStatus.AVAILABLE
             backup.save()
-            self.db.volume_update(context, volume_id,
-                                  {'status': 'error_restoring'})
+            self.db.volume_update(
+                context, volume_id,
+                {'status':
+                 (fields.VolumeStatus.ERROR if
+                  volume_previous_status == fields.VolumeStatus.CREATING else
+                  fields.VolumeStatus.ERROR_RESTORING)})
             raise exception.InvalidVolume(reason=err)
 
         expected_status = fields.BackupStatus.RESTORING
@@ -552,7 +573,8 @@ class BackupManager(manager.ThreadPoolManager):
                    {'expected_status': expected_status,
                     'actual_status': actual_status})
             self._update_backup_error(backup, err)
-            self.db.volume_update(context, volume_id, {'status': 'error'})
+            self.db.volume_update(context, volume_id,
+                                  {'status': fields.VolumeStatus.ERROR})
             raise exception.InvalidBackup(reason=err)
 
         if volume['size'] > backup['size']:
@@ -564,36 +586,52 @@ class BackupManager(manager.ThreadPoolManager):
                       'backup_id': backup['id'],
                       'backup_size': backup['size']})
 
-        backup_service = self._map_service_to_driver(backup['service'])
-        configured_service = self.driver_name
         if not self._is_our_backup(backup):
             err = _('Restore backup aborted, the backup service currently'
                     ' configured [%(configured_service)s] is not the'
                     ' backup service that was used to create this'
                     ' backup [%(backup_service)s].') % {
-                'configured_service': configured_service,
-                'backup_service': backup_service,
+                'configured_service': self.driver_name,
+                'backup_service': backup.service,
             }
             backup.status = fields.BackupStatus.AVAILABLE
             backup.save()
-            self.db.volume_update(context, volume_id, {'status': 'error'})
+            self.db.volume_update(context, volume_id,
+                                  {'status': fields.VolumeStatus.ERROR})
             raise exception.InvalidBackup(reason=err)
 
+        canceled = False
         try:
             self._run_restore(context, backup, volume)
+        except exception.BackupRestoreCancel:
+            canceled = True
         except Exception:
             with excutils.save_and_reraise_exception():
-                self.db.volume_update(context, volume_id,
-                                      {'status': 'error_restoring'})
+                self.db.volume_update(
+                    context, volume_id,
+                    {'status': (fields.VolumeStatus.ERROR if
+                                actual_status == fields.VolumeStatus.CREATING
+                                else fields.VolumeStatus.ERROR_RESTORING)})
                 backup.status = fields.BackupStatus.AVAILABLE
                 backup.save()
 
-        self.db.volume_update(context, volume_id, {'status': 'available'})
+        if canceled:
+            volume.status = fields.VolumeStatus.ERROR
+        else:
+            volume.status = fields.VolumeStatus.AVAILABLE
+            # NOTE(tommylikehu): If previous status is 'creating', this is
+            # just a new created volume and we need update the 'launched_at'
+            # attribute as well.
+            if volume_previous_status == fields.VolumeStatus.CREATING:
+                volume['launched_at'] = timeutils.utcnow()
+        volume.save()
         backup.status = fields.BackupStatus.AVAILABLE
         backup.save()
-        LOG.info('Restore backup finished, backup %(backup_id)s restored'
-                 ' to volume %(volume_id)s.',
-                 {'backup_id': backup.id, 'volume_id': volume_id})
+        LOG.info('%(result)s restoring backup %(backup_id)s to volume '
+                 '%(volume_id)s.',
+                 {'result': 'Canceled' if canceled else 'Finished',
+                  'backup_id': backup.id,
+                  'volume_id': volume_id})
         self._notify_about_backup_usage(context, backup, "restore.end")
 
     def _run_restore(self, context, backup, volume):
@@ -611,21 +649,29 @@ class BackupManager(manager.ThreadPoolManager):
         # with native threads proxy-wrapping the device file object.
         try:
             device_path = attach_info['device']['path']
+            open_mode = 'rb+' if os.name == 'nt' else 'wb'
             if (isinstance(device_path, six.string_types) and
                     not os.path.isdir(device_path)):
                 if secure_enabled:
-                    with open(device_path, 'wb') as device_file:
+                    with open(device_path, open_mode) as device_file:
                         backup_service.restore(backup, volume.id,
                                                tpool.Proxy(device_file))
                 else:
                     with utils.temporary_chown(device_path):
-                        with open(device_path, 'wb') as device_file:
+                        with open(device_path, open_mode) as device_file:
                             backup_service.restore(backup, volume.id,
                                                    tpool.Proxy(device_file))
             # device_path is already file-like so no need to open it
             else:
                 backup_service.restore(backup, volume.id,
                                        tpool.Proxy(device_path))
+        except exception.BackupRestoreCancel:
+            raise
+        except Exception:
+            LOG.exception('Restoring backup %(backup_id)s to volume '
+                          '%(volume_id)s failed.', {'backup_id': backup.id,
+                                                    'volume_id': volume.id})
+            raise
         finally:
             self._detach_device(context, attach_info, volume, properties,
                                 force=True)
@@ -696,19 +742,17 @@ class BackupManager(manager.ThreadPoolManager):
             self._update_backup_error(backup, err, status)
             raise exception.InvalidBackup(reason=err)
 
-        backup_service = self._map_service_to_driver(backup['service'])
-        if backup_service is not None:
-            configured_service = self.driver_name
-            if not self._is_our_backup(backup):
-                err = _('Delete backup aborted, the backup service currently'
-                        ' configured [%(configured_service)s] is not the'
-                        ' backup service that was used to create this'
-                        ' backup [%(backup_service)s].')\
-                    % {'configured_service': configured_service,
-                       'backup_service': backup_service}
-                self._update_backup_error(backup, err)
-                raise exception.InvalidBackup(reason=err)
+        if not self._is_our_backup(backup):
+            err = _('Delete backup aborted, the backup service currently'
+                    ' configured [%(configured_service)s] is not the'
+                    ' backup service that was used to create this'
+                    ' backup [%(backup_service)s].')\
+                % {'configured_service': self.driver_name,
+                   'backup_service': backup.service}
+            self._update_backup_error(backup, err)
+            raise exception.InvalidBackup(reason=err)
 
+        if backup.service:
             try:
                 backup_service = self.get_backup_driver(context)
                 backup_service.delete_backup(backup)
@@ -788,15 +832,13 @@ class BackupManager(manager.ThreadPoolManager):
             raise exception.InvalidBackup(reason=err)
 
         backup_record = {'backup_service': backup.service}
-        backup_service = self._map_service_to_driver(backup.service)
-        configured_service = self.driver_name
         if not self._is_our_backup(backup):
             err = (_('Export record aborted, the backup service currently '
                      'configured [%(configured_service)s] is not the '
                      'backup service that was used to create this '
                      'backup [%(backup_service)s].') %
-                   {'configured_service': configured_service,
-                    'backup_service': backup_service})
+                   {'configured_service': self.driver_name,
+                    'backup_service': backup.service})
             raise exception.InvalidBackup(reason=err)
 
         # Call driver to create backup description string
@@ -943,23 +985,22 @@ class BackupManager(manager.ThreadPoolManager):
                  {'backup_id': backup.id,
                   'status': status})
 
-        backup_service_name = self._map_service_to_driver(backup.service)
-        LOG.info('Backup service: %s.', backup_service_name)
-        if backup_service_name is not None:
-            configured_service = self.driver_name
-            if not self._is_our_backup(backup):
-                err = _('Reset backup status aborted, the backup service'
-                        ' currently configured [%(configured_service)s] '
-                        'is not the backup service that was used to create'
-                        ' this backup [%(backup_service)s].') % \
-                    {'configured_service': configured_service,
-                     'backup_service': backup_service_name}
-                raise exception.InvalidBackup(reason=err)
+        LOG.info('Backup service: %s.', backup.service)
+        if not self._is_our_backup(backup):
+            err = _('Reset backup status aborted, the backup service'
+                    ' currently configured [%(configured_service)s] '
+                    'is not the backup service that was used to create'
+                    ' this backup [%(backup_service)s].') % \
+                {'configured_service': self.driver_name,
+                 'backup_service': backup.service}
+            raise exception.InvalidBackup(reason=err)
+
+        if backup.service is not None:
             # Verify backup
             try:
                 # check whether the backup is ok or not
-                if (status == fields.BackupStatus.AVAILABLE
-                        and backup['status'] != fields.BackupStatus.RESTORING):
+                if (status == fields.BackupStatus.AVAILABLE and
+                        backup['status'] != fields.BackupStatus.RESTORING):
                     # check whether we could verify the backup is ok or not
                     backup_service = self.get_backup_driver(context)
                     if isinstance(backup_service,
@@ -1084,7 +1125,8 @@ class BackupManager(manager.ThreadPoolManager):
             protocol,
             use_multipath=use_multipath,
             device_scan_attempts=device_scan_attempts,
-            conn=conn)
+            conn=conn,
+            expect_raw_disk=True)
         vol_handle = connector.connect_volume(conn['data'])
 
         return {'conn': conn, 'device': vol_handle, 'connector': connector}
