@@ -22,7 +22,6 @@ SHOULD include dedicated exception logging.
 
 """
 
-from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_versionedobjects import exception as obj_exc
 import six
@@ -35,19 +34,6 @@ from cinder.i18n import _
 
 LOG = logging.getLogger(__name__)
 
-# TODO(smcginnis) Remove in Rocky
-exc_log_opts = [
-    cfg.BoolOpt('fatal_exception_format_errors',
-                default=False,
-                help='Make exception message format errors fatal.',
-                deprecated_for_removal=True,
-                deprecated_since='12.0.0',
-                deprecated_reason='This is only used for internal testing.'),
-]
-
-CONF = cfg.CONF
-CONF.register_opts(exc_log_opts)
-
 
 class ConvertedException(webob.exc.WSGIHTTPException):
     def __init__(self, code=500, title="", explanation=""):
@@ -160,7 +146,11 @@ class VolumeDriverException(CinderException):
 
 
 class BackupDriverException(CinderException):
-    message = _("Backup driver reported an error: %(message)s")
+    message = _("Backup driver reported an error: %(reason)s")
+
+
+class BackupRestoreCancel(CinderException):
+    message = _("Canceled backup %(back_id)s restore on volume %(vol_id)s")
 
 
 class GlanceConnectionFailed(CinderException):
@@ -222,6 +212,10 @@ class InvalidAvailabilityZone(Invalid):
     message = _("Availability zone '%(az)s' is invalid.")
 
 
+class InvalidTypeAvailabilityZones(Invalid):
+    message = _("Volume type's availability zones are invalid %(az)s.")
+
+
 class InvalidVolumeType(Invalid):
     message = _("Invalid volume type: %(reason)s")
 
@@ -450,6 +444,15 @@ class InvalidImageRef(Invalid):
     message = _("Invalid image href %(image_href)s.")
 
 
+class InvalidSignatureImage(Invalid):
+    message = _("Signature metadata is incomplete for image: "
+                "%(image_id)s.")
+
+
+class ImageSignatureVerificationException(CinderException):
+    message = _("Failed to verify image signature, reason: %(reason)s.")
+
+
 class ImageNotFound(NotFound):
     message = _("Image %(image_id)s could not be found.")
 
@@ -724,10 +727,6 @@ class GlanceMetadataNotFound(NotFound):
     message = _("Glance metadata for volume/snapshot %(id)s cannot be found.")
 
 
-class ImageDownloadFailed(CinderException):
-    message = _("Failed to download image %(image_href)s, reason: %(reason)s")
-
-
 class ExportFailure(Invalid):
     message = _("Failed to export for volume: %(reason)s")
 
@@ -1179,7 +1178,7 @@ class BadHTTPResponseStatus(VolumeDriverException):
 
 
 class BadResetResourceStatus(CinderException):
-    message = _("Bad reset resource status : %(message)s")
+    message = _("Bad reset resource status : %(reason)s")
 
 
 # ZADARA STORAGE VPSA driver exception
@@ -1286,7 +1285,7 @@ class NotSupportedOperation(Invalid):
 
 # NexentaStor driver exception
 class NexentaException(VolumeDriverException):
-    message = "%(message)s"
+    message = "%(reason)s"
 
 
 # Google Cloud Storage(GCS) backup driver
@@ -1335,7 +1334,7 @@ class InvalidAttachment(Invalid):
 
 # Veritas driver
 class UnableToExecuteHyperScaleCmd(VolumeDriverException):
-    message = _("Failed HyperScale command for '%(message)s'")
+    message = _("Failed HyperScale command for '%(command)s'")
 
 
 class UnableToProcessHyperScaleCmdOutput(VolumeDriverException):
@@ -1355,10 +1354,6 @@ class ErrorInHyperScaleVersion(VolumeDriverException):
     message = _("Error in getting HyperScale version '%(cmd_error)s'")
 
 
-class ErrorInParsingArguments(VolumeDriverException):
-    message = _("Error in parsing message arguments : Invalid Payload")
-
-
 # GPFS driver
 class GPFSDriverUnsupportedOperation(VolumeBackendAPIException):
     message = _("GPFS driver unsupported operation: %(msg)s")
@@ -1372,3 +1367,17 @@ class ServiceUserTokenNoAuth(CinderException):
     message = _("The [service_user] send_service_user_token option was "
                 "requested, but no service auth could be loaded. Please check "
                 "the [service_user] configuration section.")
+
+
+class UnsupportedNVMETProtocol(Invalid):
+    message = _("An invalid 'target_protocol' "
+                "value was provided: %(protocol)s")
+
+
+# NVMET driver
+class NVMETTargetAddError(CinderException):
+    message = "Failed to add subsystem: %(subsystem)s"
+
+
+class NVMETTargetDeleteError(CinderException):
+    message = "Failed to delete subsystem: %(subsystem)s"
