@@ -16,7 +16,6 @@
 
 import datetime
 import itertools
-import six
 
 import ddt
 import glanceclient.exc
@@ -131,11 +130,46 @@ class TestGlanceImageService(test.TestCase):
         fixture.update(kwargs)
         return fixture
 
+    @staticmethod
+    def _make_image_member_fixtures(**kwargs):
+        fixtures = []
+        fixture = {'status': None,
+                   'image_id': None,
+                   'member_id': None,
+                   'created_at': '2018-03-14T21:48:13Z',
+                   'updated_at': '2018-03-14T21:50:51Z',
+                   'schema': '/v2/schemas/member'}
+        fixture.update(kwargs)
+        fixtures.append(fixture)
+        return fixtures
+
     def _make_datetime_fixture(self):
         return self._make_fixture(created_at=self.NOW_GLANCE_FORMAT,
                                   updated_at=self.NOW_GLANCE_FORMAT,
                                   deleted_at=self.NOW_GLANCE_FORMAT)
 
+    def test_list_members(self):
+        fixture = {'status': None,
+                   'image_id': None,
+                   'member_id': None,
+                   'created_at': '2018-03-14T21:48:13Z',
+                   'updated_at': '2018-03-14T21:50:51Z',
+                   'schema': '/v2/schemas/member'}
+        image_id = '97c1ef11-3a64-4756-9f8c-7f9fb5abe09f'
+        member_id = '50fcc79f25524744a2c34682a1a74914'
+        fixture['status'] = 'accepted'
+        fixture['image_id'] = image_id
+        fixture['member_id'] = member_id
+        with mock.patch.object(self.service, '_client') as client_mock:
+            client_mock.call.return_value = self._make_image_member_fixtures(
+                image_id=image_id, member_id=member_id, status='accepted')
+            result = self.service.list_members(self.context, image_id)
+        self.assertEqual([fixture], result)
+        client_mock.call.assert_called_once_with(self.context,
+                                                 'list',
+                                                 controller='image_members',
+                                                 image_id=image_id)
+
     def test_get_api_servers(self):
         result = glance.get_api_servers(self.context)
         expected = (u'example.com:9292', False)
@@ -216,6 +250,106 @@ class TestGlanceImageService(test.TestCase):
         actual = self.service.show(self.context, image_id)
         self.assertDictEqual(expected, actual)
 
+    def test_show_shared_image_membership_success(self):
+        """Test Create Shared Image Membership Success
+
+        Ensure we can get access to a shared image
+        """
+        fixture = {'name': 'test image', 'is_public': False,
+                   'protected': False, 'visibility': 'shared'}
+        # pid = self.context.project_id
+        image_id = self.service.create(self.context, fixture)['id']
+        image = {
+            'id': image_id,
+            'name': 'test image',
+            'protected': False,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'is_public': False},
+            'owner': None,
+            'visibility': None,
+        }
+        member_id = '50fcc79f25524744a2c34682a1a74914'
+        with mock.patch.object(self.service, '_client') as client_mock:
+            with mock.patch.object(
+                    self.service, '_translate_from_glance') as tg_mock:
+                tg_mock.return_value = {}
+                mock_image = mock.Mock()
+                mock_image.is_public = False
+                mock_image.properties = {'is_public': False}
+                mock_image.visibility = 'shared'
+                mock_image.keys.return_value = image.keys()
+                client_mock.call.side_effect = [
+                    mock_image,
+                    self._make_image_member_fixtures(image_id=image_id,
+                                                     member_id=member_id,
+                                                     status='accepted')]
+                self.context.project_id = member_id
+                self.context.is_admin = False
+                self.context.user_id = image_id
+                self.context.auth_token = False
+                self.service.show(self.context, image_id)
+
+    def test_show_shared_image_membership_fail_status(self):
+        """Test Create Shared Image Membership Failure
+
+        Ensure we can't get access to a shared image with the wrong membership
+        status (in this case 'pending')
+        """
+        fixture = {'name': 'test image', 'is_public': False,
+                   'protected': False, 'visibility': 'shared'}
+        # pid = self.context.project_id
+        image_id = self.service.create(self.context, fixture)['id']
+        image = {
+            'id': image_id,
+            'name': 'test image',
+            'protected': False,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'is_public': False},
+            'owner': None,
+            'visibility': None,
+        }
+        member_id = '50fcc79f25524744a2c34682a1a74914'
+        with mock.patch.object(self.service, '_client') as client_mock:
+            with mock.patch.object(
+                    self.service, '_translate_from_glance') as tg_mock:
+                tg_mock.return_value = {}
+                mock_image = mock.Mock()
+                mock_image.is_public = False
+                mock_image.properties = {'is_public': False}
+                mock_image.visibility = 'shared'
+                mock_image.keys.return_value = image.keys()
+                client_mock.call.side_effect = [
+                    mock_image,
+                    self._make_image_member_fixtures(image_id=image_id,
+                                                     member_id=member_id,
+                                                     status='pending')]
+                self.context.project_id = member_id
+                self.context.is_admin = False
+                self.context.user_id = image_id
+                self.context.auth_token = False
+                self.assertRaises(exception.ImageNotFound,
+                                  self.service.show,
+                                  self.context,
+                                  image_id)
+
     def test_create(self):
         fixture = self._make_fixture(name='test image')
         num_images = len(self.service.detail(self.context))
@@ -529,20 +663,6 @@ class TestGlanceImageService(test.TestCase):
         self.flags(glance_num_retries=1)
         service.download(self.context, image_id, writer)
 
-    def test_download_no_data(self):
-        class MyGlanceStubClient(glance_stubs.StubGlanceClient):
-            """Returns None instead of an iterator."""
-            def data(self, image_id):
-                return None
-
-        client = MyGlanceStubClient()
-        service = self._create_image_service(client)
-        image_id = 'fake-image-uuid'
-        e = self.assertRaises(exception.ImageDownloadFailed, service.download,
-                              self.context, image_id)
-        self.assertIn('image contains no data', six.text_type(e))
-        self.assertIn(image_id, six.text_type(e))
-
     def test_client_forbidden_converts_to_imagenotauthed(self):
         class MyGlanceStubClient(glance_stubs.StubGlanceClient):
             """A client that raises a Forbidden exception."""
@@ -852,7 +972,8 @@ class TestGlanceImageServiceClient(test.TestCase):
 
         config_options = {'insecure': False,
                           'cacert': None,
-                          'timeout': None}
+                          'timeout': None,
+                          'split_loggers': False}
 
         mock_get_auth_plugin.return_value = context._ContextAuthPlugin
         mock_load.return_value = session.Session
@@ -883,7 +1004,8 @@ class TestGlanceImageServiceClient(test.TestCase):
 
         config_options = {'insecure': False,
                           'cacert': '/opt/stack/data/ca-bundle.pem',
-                          'timeout': 60}
+                          'timeout': 60,
+                          'split_loggers': False}
 
         mock_get_auth_plugin.return_value = context._ContextAuthPlugin
         mock_load.return_value = session.Session
