@@ -35,43 +35,43 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
         ctxt = context.get_admin_context()
         self.assertRaises(exception.VolumeNotFound,
                           db.volume_glance_metadata_create,
-                          ctxt, fake.volume_id, 'key1', 'value1')
+                          ctxt, fake.VOLUME_ID, 'key1', 'value1')
         self.assertRaises(exception.VolumeNotFound,
-                          db.volume_glance_metadata_get, ctxt, fake.volume_id)
-        db.volume_glance_metadata_delete_by_volume(ctxt, fake.volume2_id)
+                          db.volume_glance_metadata_get, ctxt, fake.VOLUME_ID)
+        db.volume_glance_metadata_delete_by_volume(ctxt, fake.VOLUME2_ID)
 
     def test_vol_update_glance_metadata(self):
         ctxt = context.get_admin_context()
-        db.volume_create(ctxt, {'id': fake.volume_id})
-        db.volume_create(ctxt, {'id': fake.volume2_id})
-        db.volume_glance_metadata_create(ctxt, fake.volume_id, 'key1',
+        db.volume_create(ctxt, {'id': fake.VOLUME_ID})
+        db.volume_create(ctxt, {'id': fake.VOLUME2_ID})
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME_ID, 'key1',
                                          'value1')
-        db.volume_glance_metadata_create(ctxt, fake.volume2_id, 'key1',
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME2_ID, 'key1',
                                          'value1')
-        db.volume_glance_metadata_create(ctxt, fake.volume2_id, 'key2',
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME2_ID, 'key2',
                                          'value2')
-        db.volume_glance_metadata_create(ctxt, fake.volume2_id, 'key3', 123)
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME2_ID, 'key3', 123)
 
-        expected_metadata_1 = {'volume_id': fake.volume_id,
+        expected_metadata_1 = {'volume_id': fake.VOLUME_ID,
                                'key': 'key1',
                                'value': 'value1'}
 
-        metadata = db.volume_glance_metadata_get(ctxt, fake.volume_id)
+        metadata = db.volume_glance_metadata_get(ctxt, fake.VOLUME_ID)
         self.assertEqual(1, len(metadata))
         for key, value in expected_metadata_1.items():
             self.assertEqual(value, metadata[0][key])
 
-        expected_metadata_2 = ({'volume_id': fake.volume2_id,
+        expected_metadata_2 = ({'volume_id': fake.VOLUME2_ID,
                                 'key': 'key1',
                                 'value': 'value1'},
-                               {'volume_id': fake.volume2_id,
+                               {'volume_id': fake.VOLUME2_ID,
                                 'key': 'key2',
                                 'value': 'value2'},
-                               {'volume_id': fake.volume2_id,
+                               {'volume_id': fake.VOLUME2_ID,
                                 'key': 'key3',
                                 'value': '123'})
 
-        metadata = db.volume_glance_metadata_get(ctxt, fake.volume2_id)
+        metadata = db.volume_glance_metadata_get(ctxt, fake.VOLUME2_ID)
         self.assertEqual(3, len(metadata))
         for expected, meta in zip(expected_metadata_2, metadata):
             for key, value in expected.items():
@@ -79,32 +79,32 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
 
         self.assertRaises(exception.GlanceMetadataExists,
                           db.volume_glance_metadata_create,
-                          ctxt, fake.volume_id, 'key1', 'value1a')
+                          ctxt, fake.VOLUME_ID, 'key1', 'value1a')
 
-        metadata = db.volume_glance_metadata_get(ctxt, fake.volume_id)
+        metadata = db.volume_glance_metadata_get(ctxt, fake.VOLUME_ID)
         self.assertEqual(1, len(metadata))
         for key, value in expected_metadata_1.items():
             self.assertEqual(value, metadata[0][key])
 
     def test_vols_get_glance_metadata(self):
         ctxt = context.get_admin_context()
-        db.volume_create(ctxt, {'id': fake.volume_id})
-        db.volume_create(ctxt, {'id': fake.volume2_id})
+        db.volume_create(ctxt, {'id': fake.VOLUME_ID})
+        db.volume_create(ctxt, {'id': fake.VOLUME2_ID})
         db.volume_create(ctxt, {'id': '3'})
-        db.volume_glance_metadata_create(ctxt, fake.volume_id, 'key1',
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME_ID, 'key1',
                                          'value1')
-        db.volume_glance_metadata_create(ctxt, fake.volume2_id, 'key2',
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME2_ID, 'key2',
                                          'value2')
-        db.volume_glance_metadata_create(ctxt, fake.volume2_id, 'key22',
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME2_ID, 'key22',
                                          'value22')
 
         metadata = db.volume_glance_metadata_get_all(ctxt)
         self.assertEqual(3, len(metadata))
-        self._assert_metadata_equals(fake.volume_id, 'key1', 'value1',
+        self._assert_metadata_equals(fake.VOLUME_ID, 'key1', 'value1',
                                      metadata[0])
-        self._assert_metadata_equals(fake.volume2_id, 'key2', 'value2',
+        self._assert_metadata_equals(fake.VOLUME2_ID, 'key2', 'value2',
                                      metadata[1])
-        self._assert_metadata_equals(fake.volume2_id, 'key22', 'value22',
+        self._assert_metadata_equals(fake.VOLUME2_ID, 'key22', 'value22',
                                      metadata[2])
 
     def _assert_metadata_equals(self, volume_id, key, value, observed):
@@ -114,23 +114,23 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
 
     def test_vol_delete_glance_metadata(self):
         ctxt = context.get_admin_context()
-        db.volume_create(ctxt, {'id': fake.volume_id})
-        db.volume_glance_metadata_delete_by_volume(ctxt, fake.volume_id)
-        db.volume_glance_metadata_create(ctxt, fake.volume_id, 'key1',
+        db.volume_create(ctxt, {'id': fake.VOLUME_ID})
+        db.volume_glance_metadata_delete_by_volume(ctxt, fake.VOLUME_ID)
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME_ID, 'key1',
                                          'value1')
-        db.volume_glance_metadata_delete_by_volume(ctxt, fake.volume_id)
+        db.volume_glance_metadata_delete_by_volume(ctxt, fake.VOLUME_ID)
         self.assertRaises(exception.GlanceMetadataNotFound,
-                          db.volume_glance_metadata_get, ctxt, fake.volume_id)
+                          db.volume_glance_metadata_get, ctxt, fake.VOLUME_ID)
 
     def test_vol_glance_metadata_copy_to_snapshot(self):
         ctxt = context.get_admin_context()
-        db.volume_create(ctxt, {'id': fake.volume_id})
-        snap = objects.Snapshot(ctxt, volume_id=fake.volume_id)
+        db.volume_create(ctxt, {'id': fake.VOLUME_ID})
+        snap = objects.Snapshot(ctxt, volume_id=fake.VOLUME_ID)
         snap.create()
-        db.volume_glance_metadata_create(ctxt, fake.volume_id, 'key1',
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME_ID, 'key1',
                                          'value1')
         db.volume_glance_metadata_copy_to_snapshot(ctxt, snap.id,
-                                                   fake.volume_id)
+                                                   fake.VOLUME_ID)
 
         expected_meta = {'snapshot_id': snap.id,
                          'key': 'key1',
@@ -143,19 +143,19 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
 
     def test_vol_glance_metadata_copy_from_volume_to_volume(self):
         ctxt = context.get_admin_context()
-        db.volume_create(ctxt, {'id': fake.volume_id})
-        db.volume_create(ctxt, {'id': fake.volume2_id,
-                                'source_volid': fake.volume_id})
-        db.volume_glance_metadata_create(ctxt, fake.volume_id, 'key1',
+        db.volume_create(ctxt, {'id': fake.VOLUME_ID})
+        db.volume_create(ctxt, {'id': fake.VOLUME2_ID,
+                                'source_volid': fake.VOLUME_ID})
+        db.volume_glance_metadata_create(ctxt, fake.VOLUME_ID, 'key1',
                                          'value1')
         db.volume_glance_metadata_copy_from_volume_to_volume(ctxt,
-                                                             fake.volume_id,
-                                                             fake.volume2_id)
+                                                             fake.VOLUME_ID,
+                                                             fake.VOLUME2_ID)
 
         expected_meta = {'key': 'key1',
                          'value': 'value1'}
 
-        for meta in db.volume_glance_metadata_get(ctxt, fake.volume2_id):
+        for meta in db.volume_glance_metadata_get(ctxt, fake.VOLUME2_ID):
             for (key, value) in expected_meta.items():
                 self.assertEqual(value, meta[key])
 
