@@ -30,10 +30,8 @@ HOST_NAME = 'fake.host.name'
 BACKEND_NAME = 'fake_backend_name'
 POOL_NAME = 'aggr1'
 SHARE_IP = '192.168.99.24'
-IPV6_ADDRESS = 'fe80::6e40:8ff:fe8a:130'
 EXPORT_PATH = '/fake/export/path'
 NFS_SHARE = '%s:%s' % (SHARE_IP, EXPORT_PATH)
-NFS_SHARE_IPV6 = '[%s]:%s' % (IPV6_ADDRESS, EXPORT_PATH)
 HOST_STRING = '%s@%s#%s' % (HOST_NAME, BACKEND_NAME, POOL_NAME)
 NFS_HOST_STRING = '%s@%s#%s' % (HOST_NAME, BACKEND_NAME, NFS_SHARE)
 AGGREGATE = 'aggr1'
@@ -244,7 +242,9 @@ ISCSI_TARGET_DETAILS_LIST = [
 ]
 
 IPV4_ADDRESS = '192.168.14.2'
+IPV6_ADDRESS = 'fe80::6e40:8ff:fe8a:130'
 NFS_SHARE_IPV4 = IPV4_ADDRESS + ':' + EXPORT_PATH
+NFS_SHARE_IPV6 = IPV6_ADDRESS + ':' + EXPORT_PATH
 
 RESERVED_PERCENTAGE = 7
 MAX_OVER_SUBSCRIPTION_RATIO = 19.0
