@@ -38,7 +38,6 @@ else:
     # Set storops_ex to be None for unit test
     storops_ex = None
 
-
 LOG = logging.getLogger(__name__)
 
 PROTOCOL_FC = 'FC'
@@ -58,6 +57,8 @@ class VolumeParams(object):
                              else volume.display_name)
         self._pool = None
         self._io_limit_policy = None
+        self._is_thick = None
+        self._is_compressed = None
 
     @property
     def volume_id(self):
@@ -109,11 +110,36 @@ class VolumeParams(object):
     def io_limit_policy(self, value):
         self._io_limit_policy = value
 
+    @property
+    def is_thick(self):
+        if self._is_thick is None:
+            provision = utils.get_extra_spec(self._volume, 'provisioning:type')
+            support = utils.get_extra_spec(self._volume,
+                                           'thick_provisioning_support')
+            self._is_thick = (provision == 'thick' and support == '<is> True')
+        return self._is_thick
+
+    @property
+    def is_compressed(self):
+        if self._is_compressed is None:
+            provision = utils.get_extra_spec(self._volume, 'provisioning:type')
+            compression = utils.get_extra_spec(self._volume,
+                                               'compression_support')
+            if provision == 'compressed' and compression == '<is> True':
+                self._is_compressed = True
+        return self._is_compressed
+
+    @is_compressed.setter
+    def is_compressed(self, value):
+        self._is_compressed = value
+
     def __eq__(self, other):
         return (self.volume_id == other.volume_id and
                 self.name == other.name and
                 self.size == other.size and
-                self.io_limit_policy == other.io_limit_policy)
+                self.io_limit_policy == other.io_limit_policy and
+                self.is_thick == other.is_thick and
+                self.is_compressed == other.is_compressed)
 
 
 class CommonAdapter(object):
@@ -149,8 +175,8 @@ class CommonAdapter(object):
         self.reserved_percentage = self.config.reserved_percentage
         self.max_over_subscription_ratio = (
             self.config.max_over_subscription_ratio)
-        self.volume_backend_name = (
-            self.config.safe_get('volume_backend_name') or self.driver_name)
+        self.volume_backend_name = (self.config.safe_get('volume_backend_name')
+                                    or self.driver_name)
         self.ip = self.config.san_ip
         self.username = self.config.san_login
         self.password = self.config.san_password
@@ -274,18 +300,24 @@ class CommonAdapter(object):
             'size': params.size,
             'description': params.description,
             'pool': params.pool,
-            'io_limit_policy': params.io_limit_policy}
+            'io_limit_policy': params.io_limit_policy,
+            'is_thick': params.is_thick,
+            'is_compressed': params.is_compressed
+        }
 
         LOG.info('Create Volume: %(name)s, size: %(size)s, description: '
                  '%(description)s, pool: %(pool)s, io limit policy: '
-                 '%(io_limit_policy)s.', log_params)
+                 '%(io_limit_policy)s, thick: %(is_thick)s, '
+                 '%(is_compressed)s.', log_params)
 
         return self.makeup_model(
             self.client.create_lun(name=params.name,
                                    size=params.size,
                                    pool=params.pool,
                                    description=params.description,
-                                   io_limit_policy=params.io_limit_policy))
+                                   io_limit_policy=params.io_limit_policy,
+                                   is_thin=False if params.is_thick else None,
+                                   is_compressed=params.is_compressed))
 
     def delete_volume(self, volume):
         lun_id = self.get_lun_id(volume)
@@ -335,12 +367,14 @@ class CommonAdapter(object):
         # No target info for iSCSI driver
         return []
 
-    def _detach_and_delete_host(self, host_name, lun_or_snap):
+    def _detach_and_delete_host(self, host_name, lun_or_snap,
+                                is_multiattach_to_host=False):
         @utils.lock_if(self.to_lock_host, '{lock_name}')
         def _lock_helper(lock_name):
             # Only get the host from cache here
             host = self.client.create_host_wo_lock(host_name)
-            self.client.detach(host, lun_or_snap)
+            if not is_multiattach_to_host:
+                self.client.detach(host, lun_or_snap)
             host.update()  # need update to get the latest `host_luns`
             targets = self.filter_targets_by_host(host)
             if self.remove_empty_host and not host.host_luns:
@@ -355,14 +389,16 @@ class CommonAdapter(object):
         # No return data from terminate_connection for iSCSI driver
         return {}
 
-    def _terminate_connection(self, lun_or_snap, connector):
+    def _terminate_connection(self, lun_or_snap, connector,
+                              is_multiattach_to_host=False):
         is_force_detach = connector is None
         data = {}
         if is_force_detach:
             self.client.detach_all(lun_or_snap)
         else:
-            targets = self._detach_and_delete_host(connector['host'],
-                                                   lun_or_snap)
+            targets = self._detach_and_delete_host(
+                connector['host'], lun_or_snap,
+                is_multiattach_to_host=is_multiattach_to_host)
             data = self.get_terminate_connection_info(connector, targets)
         return {
             'driver_volume_type': self.driver_volume_type,
@@ -372,7 +408,14 @@ class CommonAdapter(object):
     @cinder_utils.trace
     def terminate_connection(self, volume, connector):
         lun = self.client.get_lun(lun_id=self.get_lun_id(volume))
-        return self._terminate_connection(lun, connector)
+        # None `connector` indicates force detach, then detach all even the
+        # volume is multi-attached.
+        multiattach_flag = (connector is not None and
+                            utils.is_multiattach_to_host(
+                                volume.volume_attachment,
+                                connector['host']))
+        return self._terminate_connection(
+            lun, connector, is_multiattach_to_host=multiattach_flag)
 
     def get_connector_uids(self, connector):
         return None
@@ -404,7 +447,7 @@ class CommonAdapter(object):
             'volume_backend_name': self.volume_backend_name,
             'storage_protocol': self.protocol,
             'thin_provisioning_support': True,
-            'thick_provisioning_support': False,
+            'thick_provisioning_support': True,
             'pools': self.get_pools_stats(),
         }
 
@@ -428,9 +471,12 @@ class CommonAdapter(object):
                               {'pool_name': pool.name,
                                'array_serial': self.serial_number}),
             'thin_provisioning_support': True,
-            'thick_provisioning_support': False,
+            'thick_provisioning_support': True,
+            'compression_support': pool.is_all_flash,
             'max_over_subscription_ratio': (
-                self.max_over_subscription_ratio)}
+                self.max_over_subscription_ratio),
+            'multiattach': True
+        }
 
     def get_lun_id(self, volume):
         """Retrieves id of the volume's backing LUN.
@@ -583,7 +629,9 @@ class CommonAdapter(object):
         dest_lun = self.client.create_lun(
             name=vol_params.name, size=vol_params.size, pool=vol_params.pool,
             description=vol_params.description,
-            io_limit_policy=vol_params.io_limit_policy)
+            io_limit_policy=vol_params.io_limit_policy,
+            is_thin=False if vol_params.is_thick else None,
+            is_compressed=vol_params.is_compressed)
         src_id = src_snap.get_id()
         try:
             conn_props = cinder_utils.brick_get_connector_properties()
@@ -653,6 +701,16 @@ class CommonAdapter(object):
                 'thin clone api. source snap: %(src_snap)s, lun: %(src_lun)s.',
                 {'src_snap': src_snap.name,
                  'src_lun': 'Unknown' if src_lun is None else src_lun.name})
+        except storops_ex.UnityThinCloneNotAllowedError:
+            # Thin clone not allowed on some resources,
+            # like thick luns and their snaps
+            lun = self._dd_copy(vol_params, src_snap, src_lun=src_lun)
+            LOG.debug(
+                'Volume copied via dd because source snap/lun is not allowed '
+                'to thin clone, i.e. it is thick. source snap: %(src_snap)s, '
+                'lun: %(src_lun)s.',
+                {'src_snap': src_snap.name,
+                 'src_lun': 'Unknown' if src_lun is None else src_lun.name})
         return lun
 
     def create_volume_from_snapshot(self, volume, snapshot):
