@@ -62,18 +62,6 @@ volume_opts = [
                     'this setting is ignored if the specified file does '
                     'not exist (You can also specify \'None\' to not use '
                     'a conf file even if one exists).'),
-    cfg.FloatOpt('lvm_max_over_subscription_ratio',
-                 # This option exists to provide a default value for the
-                 # LVM driver which is different than the global default.
-                 deprecated_for_removal=True,
-                 deprecated_since="12.0.0",
-                 deprecated_reason='Oversubscription ratio should now be '
-                                   'set using the common max_over_subscription'
-                                   '_ratio config option instead.',
-                 default=None,
-                 help='max_over_subscription_ratio setting for the LVM '
-                      'driver. If set to None (the default), the general max_'
-                      'over_subscription_ratio is used.'),
     cfg.BoolOpt('lvm_suppress_fd_warnings',
                 default=False,
                 help='Suppress leaked file descriptor warnings in LVM '
@@ -122,10 +110,6 @@ class LVMVolumeDriver(driver.VolumeDriver):
         self.protocol = self.target_driver.protocol
         self._sparse_copy_volume = False
 
-        if self.configuration.lvm_max_over_subscription_ratio is not None:
-            self.configuration.max_over_subscription_ratio = \
-                self.configuration.lvm_max_over_subscription_ratio
-
     def _sizestr(self, size_in_g):
         return '%sg' % size_in_g
 
@@ -855,18 +839,13 @@ class LVMVolumeDriver(driver.VolumeDriver):
         # we need to do here is check if there is more than one attachment for
         # the volume, if there is; let the caller know that they should NOT
         # remove the export.
+        has_shared_connections = False
+        if len(volume.volume_attachment) > 1:
+            has_shared_connections = True
+
         # NOTE(jdg): For the TGT driver this is a noop, for LIO this removes
         # the initiator IQN from the targets access list, so we're good
-        # NOTE(lyarwood): Given the above note we should only call
-        # terminate_connection for the target lioadm driver when there is only
-        # one attachment left for the host specified by the connector to
-        # remove, otherwise the ACL will be removed prematurely while other
-        # attachments on the same host are still accessing the volume.
-        attachments = volume.volume_attachment
-        if volume.multiattach:
-            if sum(1 for a in attachments if a.connector and
-                    a.connector['initiator'] == connector['initiator']) > 1:
-                return True
 
-        self.target_driver.terminate_connection(volume, connector, **kwargs)
-        return len(attachments) > 1
+        self.target_driver.terminate_connection(volume, connector,
+                                                **kwargs)
+        return has_shared_connections
