@@ -152,6 +152,12 @@ class NetAppESeriesLibrary(object):
         if self.configuration.netapp_enable_multiattach:
             self._ensure_multi_attach_host_group_exists()
 
+        # This driver has been marked 'deprecated' in the Rocky release and
+        # will be removed in Stein.
+        msg = _("The NetApp E-Series driver is deprecated and will be "
+                "removed in a future release.")
+        versionutils.report_deprecated_feature(LOG, msg)
+
     def _create_rest_client(self, configuration):
         port = configuration.netapp_server_port
         scheme = configuration.netapp_transport_type.lower()
@@ -1286,19 +1292,19 @@ class NetAppESeriesLibrary(object):
                   msg_fmt)
 
         iscsi_details = self._get_iscsi_service_details()
-        iscsi_portals = self._get_iscsi_portals_for_vol(eseries_vol,
-                                                        iscsi_details)
+        iscsi_portal = self._get_iscsi_portal_for_vol(eseries_vol,
+                                                      iscsi_details)
         LOG.debug("Successfully fetched target details for volume %(id)s and "
                   "initiator %(initiator_name)s.", msg_fmt)
-        iqns = [portal['iqn'] for portal in iscsi_portals]
-        addresses = [portal['ip'] for portal in iscsi_portals]
-        ports = [portal['tcp_port'] for portal in iscsi_portals]
+        iqn = iscsi_portal['iqn']
+        address = iscsi_portal['ip']
+        port = iscsi_portal['tcp_port']
         properties = na_utils.get_iscsi_connection_properties(lun_id, volume,
-                                                              iqns, addresses,
-                                                              ports)
+                                                              iqn, address,
+                                                              port)
         if self.configuration.use_chap_auth:
             if self._client.features.CHAP_AUTHENTICATION:
-                chap_username, chap_password = self._configure_chap(iqns[0])
+                chap_username, chap_password = self._configure_chap(iqn)
                 properties['data']['auth_username'] = chap_username
                 properties['data']['auth_password'] = chap_password
                 properties['data']['auth_method'] = 'CHAP'
@@ -1353,15 +1359,13 @@ class NetAppESeriesLibrary(object):
                 msg % self._client.get_system_id())
         return ports
 
-    def _get_iscsi_portals_for_vol(self, volume, portals, anyController=True):
+    def _get_iscsi_portal_for_vol(self, volume, portals, anyController=True):
         """Get the iscsi portal info relevant to volume."""
-        manager = volume.get('currentManager')
-        current_manager_portals = [portal for portal in portals
-                                   if portal.get('controller') == manager]
-        if current_manager_portals:
-            return current_manager_portals
+        for portal in portals:
+            if portal.get('controller') == volume.get('currentManager'):
+                return portal
         if anyController and portals:
-            return portals
+            return portals[0]
         msg = _('No good iscsi portal found in supplied list for %s.')
         raise exception.NetAppDriverException(
             msg % self._client.get_system_id())
