@@ -102,9 +102,9 @@ RBD_OPTS = [
                     'value is used.'),
     cfg.BoolOpt('report_dynamic_total_capacity', default=True,
                 help='Set to True for driver to report total capacity as a '
-                     'dynamic value -used + current free- and to False to '
-                     'report a static value -quota max bytes if defined and '
-                     'global size of cluster if not-.'),
+                     'dynamic value (used + current free) and to False to '
+                     'report a static value (quota max bytes if defined and '
+                     'global size of cluster if not).'),
     cfg.BoolOpt('rbd_exclusive_cinder_pool', default=False,
                 help="Set to True if the pool is used exclusively by Cinder. "
                      "On exclusive use driver won't query images' provisioned "
@@ -191,8 +191,9 @@ class RADOSClient(object):
 
 
 @interface.volumedriver
-class RBDDriver(driver.CloneableImageVD,
-                driver.MigrateVD, driver.ManageableVD, driver.BaseVD):
+class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
+                driver.ManageableVD, driver.ManageableSnapshotsVD,
+                driver.BaseVD):
     """Implements RADOS block device (RBD) volume commands."""
 
     VERSION = '1.2.0'
@@ -200,6 +201,8 @@ class RBDDriver(driver.CloneableImageVD,
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "Cinder_Jenkins"
 
+    SUPPORTS_ACTIVE_ACTIVE = True
+
     SYSCONFDIR = '/etc/ceph/'
 
     def __init__(self, active_backend_id=None, *args, **kwargs):
@@ -327,8 +330,10 @@ class RBDDriver(driver.CloneableImageVD,
             if timeout is None:
                 timeout = self.configuration.rados_connect_timeout
 
-            LOG.debug("connecting to %(name)s (timeout=%(timeout)s).",
-                      {'name': name, 'timeout': timeout})
+            LOG.debug("connecting to %(user)s@%(name)s (conf=%(conf)s, "
+                      "timeout=%(timeout)s).",
+                      {'user': user, 'name': name, 'conf': conf,
+                       'timeout': timeout})
 
             client = self.rados.Rados(rados_id=user,
                                       clustername=name,
@@ -406,7 +411,7 @@ class RBDDriver(driver.CloneableImageVD,
                                         client=client.cluster,
                                         ioctx=client.ioctx) as v:
                         size = v.size()
-                except (self.rbd.ImageNotFound, self.rbd.OSError):
+                except self.rbd.ImageNotFound:
                     LOG.debug("Image %s is not found.", t)
                 else:
                     total_provisioned += size
@@ -482,11 +487,19 @@ class RBDDriver(driver.CloneableImageVD,
             'free_capacity_gb': 'unknown',
             'reserved_percentage': (
                 self.configuration.safe_get('reserved_percentage')),
+            # NOTE(eharney): Do not enable multiattach for this driver.
+            # For multiattach to work correctly, the exclusive-lock
+            # feature required by ceph journaling must be disabled.
+            # This has implications for replication and other Cinder
+            # operations.
+            # Multiattach support for this driver will be investigated
+            # as multi-attach support in Cinder matures.
             'multiattach': False,
             'thin_provisioning_support': True,
             'max_over_subscription_ratio': (
                 self.configuration.safe_get('max_over_subscription_ratio')),
             'location_info': location_info,
+            'backend_state': 'down'
         }
 
         backend_name = self.configuration.safe_get('volume_backend_name')
@@ -506,6 +519,8 @@ class RBDDriver(driver.CloneableImageVD,
             if not self.configuration.safe_get('rbd_exclusive_cinder_pool'):
                 total_gbi = self._get_usage_info()
                 stats['provisioned_capacity_gb'] = total_gbi
+
+            stats['backend_state'] = 'up'
         except self.rados.Error:
             # just log and return unknown capacities and let scheduler set
             # provisioned_capacity_gb = allocated_capacity_gb
@@ -1178,8 +1193,8 @@ class RBDDriver(driver.CloneableImageVD,
             secondary_id = candidates.pop()
         return secondary_id, self._get_target_config(secondary_id)
 
-    def failover_host(self, context, volumes, secondary_id=None, groups=None):
-        """Failover to replication target."""
+    def failover(self, context, volumes, secondary_id=None, groups=None):
+        """Failover replicated volumes."""
         LOG.info('RBD driver failover started.')
         if not self._is_replication_enabled:
             raise exception.UnableToFailOver(
@@ -1194,15 +1209,35 @@ class RBDDriver(driver.CloneableImageVD,
 
         # Try to demote the volumes first
         demotion_results = self._demote_volumes(volumes)
+
         # Do the failover taking into consideration if they have been demoted
         updates = [self._failover_volume(volume, remote, is_demoted,
                                          replication_status)
                    for volume, is_demoted in zip(volumes, demotion_results)]
-        self._active_backend_id = secondary_id
-        self._active_config = remote
+
         LOG.info('RBD driver failover completed.')
         return secondary_id, updates, []
 
+    def failover_completed(self, context, secondary_id=None):
+        """Failover to replication target."""
+        LOG.info('RBD driver failover completion started.')
+        secondary_id, remote = self._get_failover_target_config(secondary_id)
+
+        self._active_backend_id = secondary_id
+        self._active_config = remote
+        LOG.info('RBD driver failover completion completed.')
+
+    def failover_host(self, context, volumes, secondary_id=None, groups=None):
+        """Failover to replication target.
+
+        This function combines calls to failover() and failover_completed() to
+        perform failover when Active/Active is not enabled.
+        """
+        active_backend_id, volume_update_list, group_update_list = (
+            self.failover(context, volumes, secondary_id, groups))
+        self.failover_completed(context, secondary_id)
+        return active_backend_id, volume_update_list, group_update_list
+
     def ensure_export(self, context, volume):
         """Synchronously recreates an export for a logical volume."""
         pass
@@ -1524,7 +1559,9 @@ class RBDDriver(driver.CloneableImageVD,
         with RADOSClient(self) as client:
             for image_name in self.RBDProxy().list(client.ioctx):
                 image_id = volume_utils.extract_id_from_volume_name(image_name)
-                with RBDVolumeProxy(self, image_name, read_only=True) as image:
+                with RBDVolumeProxy(self, image_name, read_only=True,
+                                    client=client.cluster,
+                                    ioctx=client.ioctx) as image:
                     try:
                         image_info = {
                             'reference': {'source-name': image_name},
@@ -1543,6 +1580,12 @@ class RBDDriver(driver.CloneableImageVD,
                             # image is considered to be used by client(s).
                             image_info['safe_to_manage'] = False
                             image_info['reason_not_safe'] = 'volume in use'
+                        elif image_name.endswith('.deleted'):
+                            # parent of cloned volume which marked as deleted
+                            # should not be manageable.
+                            image_info['safe_to_manage'] = False
+                            image_info['reason_not_safe'] = (
+                                'volume marked as deleted')
                         else:
                             image_info['safe_to_manage'] = True
                             image_info['reason_not_safe'] = None
@@ -1724,6 +1767,68 @@ class RBDDriver(driver.CloneableImageVD,
             snapshot_name = existing_ref['source-name']
             volume.rename_snap(utils.convert_str(snapshot_name),
                                utils.convert_str(snapshot.name))
+            if not volume.is_protected_snap(snapshot.name):
+                volume.protect_snap(snapshot.name)
+
+    def get_manageable_snapshots(self, cinder_snapshots, marker, limit, offset,
+                                 sort_keys, sort_dirs):
+        """List manageable snapshots on RBD backend."""
+        manageable_snapshots = []
+        cinder_snapshot_ids = [resource['id'] for resource in cinder_snapshots]
+
+        with RADOSClient(self) as client:
+            for image_name in self.RBDProxy().list(client.ioctx):
+                with RBDVolumeProxy(self, image_name, read_only=True,
+                                    client=client.cluster,
+                                    ioctx=client.ioctx) as image:
+                    try:
+                        for snapshot in image.list_snaps():
+                            snapshot_id = (
+                                volume_utils.extract_id_from_snapshot_name(
+                                    snapshot['name']))
+                            snapshot_info = {
+                                'reference': {'source-name': snapshot['name']},
+                                'size': int(math.ceil(
+                                    float(snapshot['size']) / units.Gi)),
+                                'cinder_id': None,
+                                'extra_info': None,
+                                'safe_to_manage': False,
+                                'reason_not_safe': None,
+                                'source_reference': {'source-name': image_name}
+                            }
+
+                            if snapshot_id in cinder_snapshot_ids:
+                                # Exclude snapshots already managed.
+                                snapshot_info['reason_not_safe'] = (
+                                    'already managed')
+                                snapshot_info['cinder_id'] = snapshot_id
+                            elif snapshot['name'].endswith('.clone_snap'):
+                                # Exclude clone snapshot.
+                                snapshot_info['reason_not_safe'] = (
+                                    'used for clone snap')
+                            elif (snapshot['name'].startswith('backup')
+                                  and '.snap.' in snapshot['name']):
+                                # Exclude intermediate snapshots created by the
+                                # Ceph backup driver.
+                                snapshot_info['reason_not_safe'] = (
+                                    'used for volume backup')
+                            else:
+                                snapshot_info['safe_to_manage'] = True
+                            manageable_snapshots.append(snapshot_info)
+                    except self.rbd.ImageNotFound:
+                        LOG.debug("Image %s is not found.", image_name)
+
+        return volume_utils.paginate_entries_list(
+            manageable_snapshots, marker, limit, offset, sort_keys, sort_dirs)
+
+    def unmanage_snapshot(self, snapshot):
+        """Removes the specified snapshot from Cinder management."""
+        with RBDVolumeProxy(self, snapshot.volume_name) as volume:
+            volume.set_snap(snapshot.name)
+            children = volume.list_children()
+            volume.set_snap(None)
+            if not children and volume.is_protected_snap(snapshot.name):
+                volume.unprotect_snap(snapshot.name)
 
     def get_backup_device(self, context, backup):
         """Get a backup device from an existing volume.
