@@ -23,6 +23,7 @@ import math
 import os
 import re
 import shutil
+import tempfile
 import time
 
 from oslo_config import cfg
@@ -36,7 +37,6 @@ from cinder import db
 from cinder import exception
 from cinder.i18n import _
 from cinder.image import image_utils
-from cinder import objects
 from cinder.objects import fields
 from cinder import utils
 from cinder.volume import configuration
@@ -682,7 +682,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
     """
 
     _VALID_IMAGE_EXTENSIONS = []
-    # The following flag may be overriden by the concrete drivers in order
+    # The following flag may be overridden by the concrete drivers in order
     # to avoid using temporary volume snapshots when creating volume clones,
     # when possible.
 
@@ -740,13 +740,6 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
             msg = _("'active' must be present when writing snap_info.")
             raise exception.RemoteFSException(msg)
 
-        if not (os.path.exists(info_path) or os.name == 'nt'):
-            # We're not managing file permissions on Windows.
-            # Plus, 'truncate' is not available.
-            self._execute('truncate', "-s0", info_path,
-                          run_as_root=self._execute_as_root)
-            self._set_rw_permissions(info_path)
-
         with open(info_path, 'w') as f:
             json.dump(snap_info, f, indent=1, sort_keys=True)
 
@@ -796,7 +789,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         # NFS snapshots
         #  It needs to run as root for volumes attached to instances, but
         #  does not when in secure mode.
-        self._execute('qemu-img', 'commit', '-d', path,
+        self._execute('qemu-img', 'commit', path,
                       run_as_root=self._execute_as_root)
         self._delete(path)
 
@@ -916,6 +909,26 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
     def _get_mount_point_base(self):
         return self.base
 
+    def _ensure_share_writable(self, path):
+        """Ensure that the Cinder user can write to the share.
+
+        If not, raise an exception.
+
+        :param path: path to test
+        :raises: RemoteFSException
+        :returns: None
+        """
+
+        prefix = '.cinder-write-test-' + str(os.getpid()) + '-'
+
+        try:
+            tempfile.NamedTemporaryFile(prefix=prefix, dir=path)
+        except OSError:
+            msg = _('Share at %(dir)s is not writable by the '
+                    'Cinder volume service. Snapshot operations will not be '
+                    'supported.') % {'dir': path}
+            raise exception.RemoteFSException(msg)
+
     def _copy_volume_to_image(self, context, volume, image_service,
                               image_meta):
         """Copy the volume to the specified image."""
@@ -972,6 +985,19 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         active_fpath = os.path.join(vol_dir, active_fname)
         return active_fpath
 
+    def _get_snapshot_backing_file(self, snapshot):
+        info_path = self._local_path_volume_info(snapshot.volume)
+        snap_info = self._read_info_file(info_path)
+        vol_dir = self._local_volume_dir(snapshot.volume)
+
+        forward_file = snap_info[snapshot.id]
+        forward_path = os.path.join(vol_dir, forward_file)
+
+        # Find the file which backs this file, which represents the point
+        # in which this snapshot was created.
+        img_info = self._qemu_img_info(forward_path)
+        return img_info.backing_file
+
     def _snapshots_exist(self, volume):
         if not volume.provider_location:
             return False
@@ -984,7 +1010,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
     def _is_volume_attached(self, volume):
         return volume.attach_status == fields.VolumeAttachStatus.ATTACHED
 
-    def _create_cloned_volume(self, volume, src_vref, context):
+    def _create_cloned_volume(self, volume, src_vref):
         LOG.info('Cloning volume %(src)s to volume %(dst)s',
                  {'src': src_vref.id,
                   'dst': volume.id})
@@ -1000,6 +1026,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         vol_attrs = ['provider_location', 'size', 'id', 'name', 'status',
                      'volume_type', 'metadata']
         Volume = collections.namedtuple('Volume', vol_attrs)
+
         volume_info = Volume(provider_location=src_vref.provider_location,
                              size=src_vref.size,
                              id=volume.id,
@@ -1010,34 +1037,25 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
 
         if (self._always_use_temp_snap_when_cloning or
                 self._snapshots_exist(src_vref)):
-            kwargs = {
-                'volume_id': src_vref.id,
-                'user_id': context.user_id,
-                'project_id': context.project_id,
-                'status': fields.SnapshotStatus.CREATING,
-                'progress': '0%',
-                'volume_size': src_vref.size,
-                'display_name': 'tmp-snap-%s' % src_vref.id,
-                'display_description': None,
-                'volume_type_id': src_vref.volume_type_id,
-                'encryption_key_id': src_vref.encryption_key_id,
-            }
-            temp_snapshot = objects.Snapshot(context=context,
-                                             **kwargs)
-            temp_snapshot.create()
+            snap_attrs = ['volume_name', 'volume_size', 'name',
+                          'volume_id', 'id', 'volume']
+            Snapshot = collections.namedtuple('Snapshot', snap_attrs)
+
+            temp_snapshot = Snapshot(volume_name=volume_name,
+                                     volume_size=src_vref.size,
+                                     name='clone-snap-%s' % src_vref.id,
+                                     volume_id=src_vref.id,
+                                     id='tmp-snap-%s' % src_vref.id,
+                                     volume=src_vref)
 
             self._create_snapshot(temp_snapshot)
             try:
                 self._copy_volume_from_snapshot(temp_snapshot,
                                                 volume_info,
                                                 volume.size)
-                # remove temp snapshot after the cloning is done
-                temp_snapshot.status = fields.SnapshotStatus.DELETING
-                temp_snapshot.context = context.elevated()
-                temp_snapshot.save()
+
             finally:
                 self._delete_snapshot(temp_snapshot)
-                temp_snapshot.destroy()
         else:
             self._copy_volume_image(self.local_path(src_vref),
                                     self.local_path(volume_info))
@@ -1091,6 +1109,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         self._validate_state(volume_status, acceptable_states)
 
         vol_path = self._local_volume_dir(snapshot.volume)
+        self._ensure_share_writable(vol_path)
 
         # Determine the true snapshot file for this snapshot
         # based on the .info file
@@ -1427,6 +1446,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
     def _create_snapshot_online(self, snapshot, backing_filename,
                                 new_snap_path):
         # Perform online snapshot via Nova
+        context = snapshot._context
+
         self._do_create_snapshot(snapshot,
                                  backing_filename,
                                  new_snap_path)
@@ -1439,7 +1460,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
 
         try:
             result = self._nova.create_volume_snapshot(
-                snapshot.obj_context,
+                context,
                 snapshot.volume_id,
                 connection_info)
             LOG.debug('nova call result: %s', result)
@@ -1454,7 +1475,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         increment = 1
         timeout = 600
         while True:
-            s = db.snapshot_get(snapshot.obj_context, snapshot.id)
+            s = db.snapshot_get(context, snapshot.id)
 
             LOG.debug('Status of snapshot %(id)s is now %(status)s',
                       {'id': snapshot['id'],
@@ -1617,8 +1638,7 @@ class RemoteFSSnapDriver(RemoteFSSnapDriverBase):
     def create_cloned_volume(self, volume, src_vref):
         """Creates a clone of the specified volume."""
 
-        return self._create_cloned_volume(volume, src_vref,
-                                          src_vref.obj_context)
+        return self._create_cloned_volume(volume, src_vref)
 
     @locked_volume_id_operation
     def copy_volume_to_image(self, context, volume, image_service, image_meta):
@@ -1659,8 +1679,7 @@ class RemoteFSSnapDriverDistributed(RemoteFSSnapDriverBase):
     def create_cloned_volume(self, volume, src_vref):
         """Creates a clone of the specified volume."""
 
-        return self._create_cloned_volume(volume, src_vref,
-                                          src_vref.obj_context)
+        return self._create_cloned_volume(volume, src_vref)
 
     @coordination.synchronized('{self.driver_prefix}-{volume.id}')
     def copy_volume_to_image(self, context, volume, image_service, image_meta):
@@ -1715,14 +1734,13 @@ class RemoteFSPoolMixin(object):
         for share in self._mounted_shares:
             (share_capacity,
              share_free,
-             share_used) = self._get_capacity_info(share)
+             total_allocated) = self._get_capacity_info(share)
 
             pool = {'pool_name': self._get_pool_name_from_share(share),
                     'total_capacity_gb': share_capacity / float(units.Gi),
                     'free_capacity_gb': share_free / float(units.Gi),
-                    'provisioned_capacity_gb': share_used / float(units.Gi),
-                    'allocated_capacity_gb': (
-                        share_capacity - share_free) / float(units.Gi),
+                    'provisioned_capacity_gb': (
+                        total_allocated / float(units.Gi)),
                     'reserved_percentage': (
                         self.configuration.reserved_percentage),
                     'max_over_subscription_ratio': (
